@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2021 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,7 +22,9 @@
 # This file contains functions used to help debug memory leaks. They aren't
 # called by default, but can be used when problems occur.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import *
+
 import time
 import weakref
 import types
@@ -35,6 +37,25 @@ import renpy
 
 memory_log = renpy.log.open("memory")
 
+# Names that are intended to be constant containers and may be skipped during profiling.
+constant_containers = {
+    "renpy.game.script",
+    "renpy.display.image.images",
+    "renpy.display.image.image_attributes",
+    "renpy.python.py_compile_cache",
+    "renpy.python.old_py_compile_cache",
+    "renpy.python.store_dicts",
+    "renpy.python.store_modules",
+    "renpy.pyanalysis.ccache",
+    "renpy.gl2.live2d.common_cache",
+    "renpy.sl2.slast.scache",
+    "renpy.sl2.slast.ccache",
+    "renpy.sl2.slparser.all_statements",
+    "renpy.screenlang.all_statements",
+    "renpy.display.screen.screens_at_sort",
+    "renpy.display.screen.screens",
+    "renpy.display.screen.screens_by_name",
+}
 
 def print_garbage(gen):
     """
@@ -82,10 +103,10 @@ def cycle_finder(o, name):
 
         paths[ido] = path
 
-        if isinstance(o, (int, float, types.NoneType, types.ModuleType, types.ClassType)):
+        if isinstance(o, (int, float, type(None), types.ModuleType, type)):
             o_repr = repr(o)
 
-        elif isinstance(o, (str, unicode)):
+        elif isinstance(o, basestring):
             if len(o) <= 80:
                 o_repr = repr(o).encode("utf-8")
             else:
@@ -98,7 +119,7 @@ def cycle_finder(o, name):
             o_repr = "<" + o.__class__.__name__ + ">"
 
         elif isinstance(o, types.MethodType):
-            o_repr = "<method {0}.{1}>".format(o.im_class.__name__, o.im_func.__name__)
+            o_repr = "<method {0}.{1}>".format(o.__self__.__class__.__name__, o.__func__.__name__)
 
         elif isinstance(o, object):
             o_repr = "<{0}>".format(type(o).__name__)
@@ -113,11 +134,11 @@ def cycle_finder(o, name):
                 visit(ido, oo, "{0}[{1!r}]".format(path, i))
 
         if isinstance(o, dict):
-            for k, v in o.iteritems():
+            for k, v in o.items():
                 visit(ido, v, "{0}[{1!r}]".format(path, k))
 
         elif isinstance(o, types.MethodType):
-            visit(ido, o.im_self, path + ".im_self")
+            visit(ido, o.__self__, path + ".im_self")
 
         else:
 
@@ -136,7 +157,7 @@ def cycle_finder(o, name):
 
             state = get(2, { })
             if isinstance(state, dict):
-                for k, v in state.iteritems():
+                for k, v in state.items():
                     visit(ido, v, path + "." + k)
             else:
                 visit(ido, state, path + ".__getstate__()")
@@ -210,11 +231,6 @@ def walk_memory(roots, seen=None):
     # A map from root_name to total_size.
     size = collections.defaultdict(int)
 
-    def add(name, o):
-        """
-        Adds o to the worklist if it's not in seen.
-        """
-
     for name, o in roots:
         id_o = id(o)
 
@@ -229,7 +245,7 @@ def walk_memory(roots, seen=None):
     get_referents = gc.get_referents
     worklist_append = worklist.append
 
-    ignore_types = (types.ModuleType, types.ClassType, types.FunctionType)
+    ignore_types = (types.ModuleType, type, types.FunctionType)
 
     while worklist:
         name, o = worklist.pop(0)
@@ -251,7 +267,7 @@ def walk_memory(roots, seen=None):
     return size, seen
 
 
-def profile_memory_common(packages=[ "renpy", "store" ]):
+def profile_memory_common(packages=[ "renpy", "store" ], skip_constants=False):
     """
     Profiles object, surface, and texture memory used in the renpy and store
     packages.
@@ -281,12 +297,17 @@ def profile_memory_common(packages=[ "renpy", "store" ]):
             continue
 
         for name, o in mod.__dict__.items():
-            roots.append((mod_name + "." + name, o))
+            name = mod_name + "." + name
+
+            if skip_constants and name in constant_containers:
+                continue
+
+            roots.append((name, o))
 
     return walk_memory(roots)
 
 
-def profile_memory(fraction=1.0, minimum=0):
+def profile_memory(fraction=1.0, minimum=0, skip_constants=False):
     """
     :doc: memory
 
@@ -307,6 +328,10 @@ def profile_memory(fraction=1.0, minimum=0):
         If a name is accounted less than `minimum` bytes of memory, it will
         not be printed.
 
+    `skip_constants`
+        If True, the profiler will skip scanning of large Ren'Py's containers,
+        that are intended to be immutable after startup.
+
     As it has to scan all memory used by Ren'Py, this function may take a
     long time to complete.
     """
@@ -316,7 +341,7 @@ def profile_memory(fraction=1.0, minimum=0):
     write("Memory profile at " + time.ctime() + ":")
     write("")
 
-    usage = [ (v, k) for (k, v) in profile_memory_common()[0].items() ]
+    usage = [ (v, k) for (k, v) in profile_memory_common(skip_constants=skip_constants)[0].items() ]
     usage.sort()
 
     # The total number of bytes allocated.
@@ -342,7 +367,7 @@ old_usage = { }
 old_total = 0
 
 
-def diff_memory(update=True):
+def diff_memory(update=True, skip_constants=False):
     """
     :doc: memory
 
@@ -355,6 +380,10 @@ def diff_memory(update=True):
     than one name, it's assigned to the name it's most directly reachable
     from.
 
+    `skip_constants`
+        If True, the profiler will skip scanning of large Ren'Py's containers,
+        that are intended to be immutable after startup.
+
     As it has to scan all memory used by Ren'Py, this function may take a
     long time to complete.
     """
@@ -367,12 +396,12 @@ def diff_memory(update=True):
     write("Memory diff at " + time.ctime() + ":")
     write("")
 
-    usage = profile_memory_common()[0]
+    usage = profile_memory_common(skip_constants=skip_constants)[0]
     total = sum(usage.values())
 
     diff = [ ]
 
-    for k, v in usage.iteritems():
+    for k, v in usage.items():
         diff.append((
             v - old_usage.get(k, 0),
             k))
@@ -424,8 +453,8 @@ def profile_rollback():
     # Walk the log, finding new roots and rollback information.
     for rb in log:
 
-        for store_name, store in rb.stores.iteritems():
-            for var_name, o in store.iteritems():
+        for store_name, store in rb.stores.items():
+            for var_name, o in store.items():
                 name = store_name + "." + var_name
                 id_o = id(o)
 
@@ -449,7 +478,7 @@ def profile_rollback():
 
     sizes = walk_memory(roots, seen)[0]
 
-    usage = [ (v, k) for (k, v) in sizes.iteritems() ]
+    usage = [ (v, k) for (k, v) in sizes.items() ]
     usage.sort()
 
     write("Total Bytes".rjust(13) + " " + "Per Rollback".rjust(13))
@@ -479,6 +508,9 @@ def find_parents(cls):
     # GC to save memory.
     gc.collect()
 
+    if gc.garbage:
+        del gc.garbage[:]
+
     objs = gc.get_objects()
 
     def print_path(o):
